@@ -17,6 +17,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <limits>
+
 #include "config.h"
 
 #include <signal.h>
@@ -84,7 +86,18 @@ extern "C" {
 
 #include <iostream>
 
-static long obsoleteDataAge = 30; // maximum ge of data in seconds until value is invalid
+// maximum age of data in seconds until it is invalid
+#if defined(HAVE_LIB_GPS)
+// Using normal libgps we don't trust situation if GPS daemon is not
+// responding in time
+static long obsoleteDataAge = 30;
+#else
+// Using liblocation on the other hand we trust it to send diffs
+// every time something relvant changes.
+// Data does not age!
+static long obsoleteDataAge = std::numeric_limits<long>::max();
+#endif
+
 static long reconnectTime   = 10; // Period of time after which we check for disconnection and
                                   // trigger reconnection
 static long refreshTime     = 10; // Period of time after which we update the stream views
@@ -420,50 +433,139 @@ double GetRelativeQualityFromDbm(double quality)
   }
 }
 
+class SatelliteObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef   font;
+  Lum::OS::ColorRef  green;
+  Lum::OS::ColorRef  grey;
+  GPSData::Satellite satellite;
+
+public:
+  SatelliteObject()
+  : font(Lum::OS::display->GetFont()),
+    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
+  {
+    // no code
+  }
+
+  void SetSatellite(const GPSData::Satellite& satellite)
+  {
+    this->satellite=satellite;
+  }
+
+  void CalcSize()
+  {
+    minWidth=Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
+             10*font->StringWidth(L"m");
+    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+              font->height+
+              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    //
+    // Signal strength bar
+    //
+
+    double percent=GetRelativeQualityFromDbm(satellite.snr)/100.0;
+
+    if (satellite.used) {
+      draw->PushForeground(green);
+    }
+    else {
+      draw->PushForeground(grey);
+    }
+
+    draw->FillRectangle(this->x,
+                        this->y+
+                        Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder),
+                        (size_t)(width*percent),height-2);
+
+
+    draw->PopForeground();
+
+    //
+    // Name of Satellite
+    //
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     Lum::Base::NumberToWString(satellite.id));
+    draw->PopForeground();
+    draw->PopFont();
+
+    //
+    //
+    // Signal strength label
+
+    Lum::OS::FontRef    font=Lum::OS::display->GetFont();
+    std::wstring        label=Lum::Base::NumberToWString(lround(percent*100))+L"%";
+    Lum::OS::FontExtent extent;
+
+    font->StringExtent(label,extent);
+
+    draw->PushForeground(Lum::OS::Display::textColor);
+    draw->PushFont(font,0);
+    draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
+                     this->y+
+                     Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font->ascent,
+                     label);
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static SatelliteObject* Create(bool horizFlex, bool vertFlex)
+  {
+    SatelliteObject *object=new SatelliteObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class SatellitesDataProvider : public SatellitesModel::DataProvider
 {
 private:
-  Lum::PercentBar                 *bar;
-  Lum::Model::DoubleDataStreamRef signal;
+  SatelliteObject *object;
 
 public:
   SatellitesDataProvider()
-  : bar(new Lum::PercentBar()),
-    signal(new Lum::Model::DoubleDataStream())
+  : object(SatelliteObject::Create(true,false))
   {
-    bar->SetFlex(true,true);
-    bar->SetModel(signal);
-
-    signal->SetNotificationMode(Lum::Model::DoubleDataStream::notifyExplicit);
-    signal->SetChannels(1);
+    // no code
   }
 
   ~SatellitesDataProvider()
   {
-    delete bar;
-  }
-
-  std::wstring GetString(const SatellitesModel::Iterator& iter, size_t column) const
-  {
-    switch (column) {
-    case 1:
-      return Lum::Base::NumberToWString(iter->id);
-    case 2:
-      return iter->used ? L"X" : L"";
-    default:
-      return L"";
-    }
+    delete object;
   }
 
   Lum::Object* GetObject(const SatellitesModel::Iterator& iter, size_t column) const
   {
-    switch (column) {
-    case 3:
-      signal->Set(0,GetRelativeQualityFromDbm(iter->snr)/100.0);
-      return bar;
-    default:
-      return NULL;
-    }
+    object->SetSatellite(*iter);
+    return object;
   }
 };
 
@@ -513,6 +615,11 @@ private:
   Lum::Base::SystemTime time;
 
 public:
+  Track()
+  {
+    // no code
+  }
+
   Track(const std::wstring& name, const Lum::Base::SystemTime& time)
   : name(name),
     time(time)
@@ -531,23 +638,106 @@ public:
   }
 };
 
+class TrackObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  Track            track;
+
+public:
+  TrackObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetTrack(const Track& track)
+  {
+    this->track=track;
+  }
+
+  void CalcSize()
+  {
+    minWidth=20*font->StringWidth(L"m")+
+             Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
+             5*font->StringWidth(L"m");
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of Track
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     track.GetName());
+
+    //
+    //
+    // Time of track
+
+    draw->DrawString(this->x+width-font->StringWidth(track.GetTimeString()),
+                     this->y+font->ascent,
+                     track.GetTimeString());
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static TrackObject* Create(bool horizFlex, bool vertFlex)
+  {
+    TrackObject *object=new TrackObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
 
 typedef Lum::Model::StdTable<Track> TracksModel;
 typedef Lum::Base::Reference<TracksModel> TracksModelRef;
 
 class TracksDataProvider : public TracksModel::DataProvider
 {
+private:
+  TrackObject *object;
+
 public:
-  std::wstring GetString(const TracksModel::Iterator& iter, size_t column) const
+  TracksDataProvider()
+   : object(TrackObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return iter->GetName();
-    case 2:
-      return iter->GetTimeString();
-    default:
-      assert(false);
-    }
+    // no code
+  }
+
+  ~TracksDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const TracksModel::Iterator& iter, size_t column) const
+  {
+    object->SetTrack(*iter);
+    return object;
   }
 };
 
@@ -646,7 +836,8 @@ private:
       device=(LocationGPSDevice*)g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
 
       if (device!=NULL) {
-        /* Install signal handlers */
+        std::cout << "Connecting callbacks..." << std::endl;
+
         idd_changed=g_signal_connect(device,"changed",G_CALLBACK(gps_location_changed),NULL);
         idd_con=g_signal_connect(device,"connected",G_CALLBACK(gps_location_connected),NULL);
         idd_disc=g_signal_connect(device,"disconnected",G_CALLBACK(gps_location_disconnected),NULL);
@@ -819,6 +1010,21 @@ public:
 
     startTrackingAction->Enable();
     stopTrackingAction->Disable();
+
+    satellites->SetEmptyText(L"(no satellites found)");
+    tracks->SetEmptyText(L"(no tracks found)");
+
+    // For testsing purposes
+    /*
+    for (int i=0; i<20; i++) {
+      GPSData::Satellite s;
+
+      s.id=i;
+      s.used=i%2!=0;
+      s.snr=20;
+
+      satellites->Append(s);
+    }*/
   }
 
   ~MyWindow()
@@ -863,19 +1069,17 @@ public:
                                                 hPanel));
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Id",Lum::Base::Size::stdCharWidth,3);
-    headerModel->AddColumn(L"Used",Lum::Base::Size::stdCharWidth,1);
-    headerModel->AddColumn(L"Signal",Lum::Base::Size::stdCharWidth,25,true);
+    headerModel->AddColumn(L"Satellite",Lum::Base::Size::stdCharWidth,25,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->SetShowHeader(true);
+    table->SetShowHeader(false);
     table->SetHeaderModel(headerModel);
     table->SetModel(satellites);
     table->GetTableView()->SetAutoFitColumns(true);
 
     multiView.AddView(Lum::Def::MultiView::View(1,
-                                                Lum::Def::Desc(L"Satelites"),
+                                                Lum::Def::Desc(L"Satellites"),
                                                 table));
 
     compass=new Compass();
@@ -898,12 +1102,11 @@ public:
                                                 ->Add(Lum::View::Create(TimeLine::Create(altitudeStream,true,true),true,true))));
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Track",Lum::Base::Size::stdCharWidth,10,true);
-    headerModel->AddColumn(L"Time",Lum::Base::Size::stdCharWidth,19); // TODO
+    headerModel->AddColumn(L"Track",Lum::Base::Size::stdCharWidth,40,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->SetShowHeader(true);
+    table->SetShowHeader(false);
     table->SetHeaderModel(headerModel);
     table->SetModel(tracks);
     table->SetSelection(trackSelection);
